Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Util.JsonSourceType of a read_ function as produced by atdgen -json.
In versions of yojson greater than 1.0.1, type Yojson.Safe.lexer_state is equivalent to Yojson.lexer_state, Yojson.Basic.lexer_state and Yojson.Raw.lexer_state.
Read a JSON value from a lexbuf.
Convert a JSON value from a string.
val from_channel :
?buf:Buffer.t ->
?fname:string ->
?lnum:int ->
'a reader ->
in_channel ->
'aRead a JSON value from a channel.
Read a JSON value from a channel.
val seq_from_lexbuf :
?fin:(unit -> unit) ->
'a reader ->
Yojson.Safe.lexer_state ->
Lexing.lexbuf ->
'a Seq.tRead a stream of JSON values from a lexbuf.
val seq_from_string :
?buf:Buffer.t ->
?fin:(unit -> unit) ->
?fname:string ->
?lnum:int ->
'a reader ->
string ->
'a Seq.tRead a stream of JSON values from a channel. Values do not have to be separated by newline characters.
val seq_from_channel :
?buf:Buffer.t ->
?fin:(unit -> unit) ->
?fname:string ->
?lnum:int ->
'a reader ->
in_channel ->
'a Seq.tRead a stream of JSON values from a channel. Values do not have to be separated by newline characters.
val seq_from_file :
?buf:Buffer.t ->
?fin:(unit -> unit) ->
?fname:string ->
?lnum:int ->
'a reader ->
string ->
'a Seq.tRead a stream of JSON values from a file. Values do not have to be separated by newline characters.
val list_from_string :
?buf:Buffer.t ->
?fin:(unit -> unit) ->
?fname:string ->
?lnum:int ->
'a reader ->
string ->
'a listRead a list of JSON values from a channel. Values do not have to be separated by newline characters.
val list_from_channel :
?buf:Buffer.t ->
?fin:(unit -> unit) ->
?fname:string ->
?lnum:int ->
'a reader ->
in_channel ->
'a listRead a list of JSON values from a channel. Values do not have to be separated by newline characters.
val list_from_file :
?buf:Buffer.t ->
?fname:string ->
?lnum:int ->
'a reader ->
string ->
'a listRead a list of JSON values from a file. Values do not have to be separated by newline characters.
Write a JSON value to a channel.
Write a stream of values to a string.
Write a stream of values to a channel.
Write a stream of values to a file.
Write a list of values to a string.
Write a list of values to a channel.
Write a list of values to a file.
preset_unknown_field_handler src_loc field_name raises a Failure exception with a message containing the location of the type definition in the source ATD file (src_loc) and the name of the field (field_name).
Function called when an unknown JSON field is encountered if the code was generated by atdgen -json-strict-fields. Its preset behavior is to call preset_unknown_field_handler which raises a Failure exception.
Usage: !Atdgen_runtime.Util.Json.unknown_field_handler src_loc field_name where src_loc is the location of the type definition in the source ATD file and field_name is the unknown JSON field name.