Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Atdgen_runtimeSourceJson adapters and tools for the user to make their own json adapters for common situations.
Various convenience types and functions meant for the user of atdgen.
This module complements the Yojson.Safe lexer and was extracted from the original source file read.mll.